/*
Tiny GLUT program to show how textures work.

Orion Sky Lawlor, olawlor@acm.org, 2006/11/09 (Public Domain)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glew.h> /* GL Extensions Wrangler, http://glew.sourceforge.net/ */
#include <GL/glut.h> /* GL Utilities Toolkit, http://freeglut.sourceforge.net/ */

void Exit(const char *where,const char *why) {
	fprintf (stderr, "FATAL OpenGL Error in %s: %s\n", where, why);
	abort();	
}

// Check GL for errors after the last command
void Check(const char *where) {
	GLenum e=glGetError();
	if (e==GL_NO_ERROR) return;
	const GLubyte *errString = gluErrorString(e);
	Exit(where, (const char *)errString);
}

// Draw a little quadrilateral onscreen
void drawBox(double sy,double ey) {
	glBegin(GL_TRIANGLE_FAN);
	glTexCoord2f(0,0);   glVertex4f(-1,sy,0,1);  
	glTexCoord2f(0,1);   glVertex4f(-1,ey+0.01,0,1);  
	glTexCoord2f(1,1);   glVertex4f(+1,ey,0,1);  
	glTexCoord2f(1,0);   glVertex4f(+1,sy,0,1);
	glEnd();
}

/******** Program Initialization *********/
int win_w, win_h; /**< Width and height (in pixels) of our window at last resize */

/* GLUT calls this routine every time our window needs to be redrawn */
void main_display(void) {
	Check("Entering display routine");

// Clear the color and depth buffers
	glClearColor(0.2,0.3,0.4,1.0); /* set clear color */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* actually clear color and depth */
	
// Draw stuff onscreen
	GLuint texNum=0;
	glGenTextures(1,&texNum); /* texNum is now a unique "texture name"--probably the value 1! */
	glActiveTextureARB(GL_TEXTURE0_ARB); /* we're using texture unit zero (this is the default, actually) */
	glBindTexture(GL_TEXTURE_2D,texNum); /* texNum is now the current texture--all texture ops now refer to texNum */

	/* Fill the currently bound texture (texNum) */
	int tex_w=128, tex_h=128;
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,tex_w,tex_h,0,GL_RGB,GL_UNSIGNED_BYTE,0);
	Check("after texture creation?");

	/* Set the magnification, minification, and wrap modes on the current texture (texNum) */
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
	
	
	GLuint fb;///< Framebuffer object
	glGenFramebuffersEXT(1, &fb); /* Make a new framebuffer object */
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb); /* set FB as the current framebuffer object */
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,   /* bind texture name "tex" as the color buffer */
        	GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D, texNum, 0);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0); /* back to default framebuffer (the screen) */

	// Here's how to draw using the framebuffer (this is fairly expensive: 35us)
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb);
	GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	if (status!=GL_FRAMEBUFFER_COMPLETE_EXT) Exit("Main","Bad framebuffer setup!");
	glViewport(0,0,tex_w,tex_h); /* set viewport to size of texture */

	glDisable(GL_TEXTURE_2D);
	glColor4f(1,0,0,1);  
	drawBox(-1,-0.8);
	glColor4f(0,1,0,1);  
	drawBox(-0.5,+0.5);
	glColor4f(0,0,1,1);
	glutSolidSphere(0.3,10,10);

	// Switch back to drawing to screen:
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	glViewport(0,0,win_w,win_h);

	// Throw away our framebuffer (fairly expensive!)
	glDeleteFramebuffersEXT(1,&fb);
	

	// Draw geometry, using currently bound texture */
	glColor4f(1,1,1,1);  
	glEnable(GL_TEXTURE_2D); /*<- fixed-function (non-GLSL) texture enable */
	drawBox(-0.9,0.9);
	

	/* Throw away all the texture data bound to texNum.  Note you can have many textures at once, so only throw out stuff you're really done using! */
	glDeleteTextures(1,&texNum);

	glBindTexture(GL_TEXTURE_2D,0);
	
// Display the stuff drawn so far to the screen:
	glutSwapBuffers();
	Check("Leaving display routine");
}

/* GLUT calls this routine every time our window changes size */
void main_reshape(int w, int h) {
	win_w=w; win_h=h;
	glViewport(0, 0, (GLsizei)w, (GLsizei)(h));
}

int main(int argc, char** argv)
{
	glutInit(&argc,argv);

	// Ask for a back buffer, color buffer, and Z buffer in a window.
	int mode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
	glutInitDisplayMode (mode);
	int main_window = glutCreateWindow("Bare-metal OpenGL Demo");
	
	// Turn on OpenGL extensions, like programmable shaders
	//   WARNING: you MUST call glutCreateWindow before doing this!
	//   If you don't call glewInit, or call it wrong, you'll segfault!
	glewInit();

	// Ask GLUT to call us when we need to redraw
	glutDisplayFunc (main_display);
	glutReshapeFunc (main_reshape);
	
	// Process events until user closes our window...
	glutMainLoop();
	
	return 0;
}
